package com.zyeeda.test;

import java.net.URL;
import java.util.Hashtable;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.persistence.Persistence;
import javax.transaction.UserTransaction;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SessionConfiguration;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import bitronix.tm.Configuration;
import bitronix.tm.TransactionManagerServices;

@Test
public class AppTest {
	
	private final static String JNDI_USER_TRANSACTION_NAME = "btmTransactionManager";
	private final static String JNDI_TRANSACTION_SYNCHRONIZATION_REGISTRY_NAME = "btmSynchronizationRegistry";
	
	@BeforeTest
    public void setUp() {
		Configuration cfg = TransactionManagerServices.getConfiguration();
		URL url = this.getClass().getClassLoader().getResource("bitronix-datasources.properties");
		cfg.setResourceConfigurationFilename(url.getPath());
		cfg.setJndiUserTransactionName(JNDI_USER_TRANSACTION_NAME);
		cfg.setJndiTransactionSynchronizationRegistryName(JNDI_TRANSACTION_SYNCHRONIZATION_REGISTRY_NAME);
		TransactionManagerServices.getTransactionManager();
    }
	
	@Test
	public void testProcessPersistence() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("META-INF/drools-changeset.xml"), ResourceType.CHANGE_SET);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.out.println(error.toString());
			}
			
			throw new IllegalArgumentException("Could not parse knowledge changeset.");
		}

		KnowledgeBase kbase = kbuilder.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		
		UserTransaction utx = this.getTransaction();
			
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, Persistence.createEntityManagerFactory("default"));
		env.set(EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager());
		env.set(EnvironmentName.TRANSACTION, utx);
		
		Properties properties = new Properties();
        properties.setProperty( "drools.commandService",
                                "org.drools.persistence.session.SingleSessionCommandService" );
        properties.setProperty( "drools.processInstanceManagerFactory",
                                "org.drools.persistence.processinstance.JPAProcessInstanceManagerFactory" );
        properties.setProperty( "drools.workItemManagerFactory",
                                "org.drools.persistence.processinstance.JPAWorkItemManagerFactory" );
        properties.setProperty( "drools.processSignalManagerFactory",
                                "org.drools.persistence.processinstance.JPASignalManagerFactory" );
		SessionConfiguration conf = new SessionConfiguration(properties);
		
		StatefulKnowledgeSession ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, conf, env);
		ProcessInstance pi = ksession.startProcess("com.zyeeda.system.TestFlow");
		pi.signalEvent("signal", "goon");
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private UserTransaction getTransaction() throws Exception {
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "bitronix.tm.jndi.BitronixInitialContextFactory");
		Context ctx = new InitialContext(env);
		
		UserTransaction utx = (UserTransaction) ctx.lookup(JNDI_USER_TRANSACTION_NAME);
		
		return utx;
	}
}
